/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/gpl-3.0.txt
 *
 */
package com.frojasg1.textcrypt.view;

import com.frojasg1.applications.common.configuration.application.BaseApplicationConfigurationInterface;
import com.frojasg1.general.StringFunctions;
import com.frojasg1.general.progress.OperationCancellation;
import com.frojasg1.general.progress.UpdatingProgress;
import com.frojasg1.textcrypt.controller.TextcryptController;
import com.hotmail.frojasg1.applications.common.components.resizecomp.MapResizeRelocateComponentItem;
import com.hotmail.frojasg1.applications.common.components.resizecomp.ResizeRelocateItem;
import java.awt.Color;
import java.util.Date;

/**
 *
 * @author axpe
 */
public class TextInfoJPanel extends javax.swing.JPanel implements UpdatingProgress
{
	BaseApplicationConfigurationInterface _appliConf = null;

	protected MapResizeRelocateComponentItem _mapRRCI = null;

	OperationCancellation _operationCancellation = null;

	Date a_beginningOfOperation = null;

	TextcryptController _controller = null;
	
	/**
	 * Creates new form TextComponentJPanel
	 */
	public TextInfoJPanel( BaseApplicationConfigurationInterface conf,
								TextcryptController controller )
	{
		_appliConf = conf;
		_controller = controller;

		initComponents();

		_mapRRCI = createResizeRelocateInfo();
	}

	public String getFileName()
	{
		return( jtf_fileName.getText() );
	}

	public void setFileName( String fileName )
	{
		if( fileName != null )
			jtf_fileName.setText( fileName );
	}

	public void setOperationCancellation( OperationCancellation oc )
	{
		_operationCancellation = oc;
	}

	public int getStrength()
	{
		return( jSlider1.getValue() );
	}

	public void setStrength( int strength )
	{
		jSlider1.setValue( strength );
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	@SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jLabel1 = new javax.swing.JLabel();
        jtf_fileName = new javax.swing.JTextField();
        jLabel2 = new javax.swing.JLabel();
        jPasswordField1 = new javax.swing.JPasswordField();
        jLabel3 = new javax.swing.JLabel();
        jPasswordField2 = new javax.swing.JPasswordField();
        jButton1 = new javax.swing.JButton();
        jProgressBar1 = new javax.swing.JProgressBar();
        jb_cancel = new javax.swing.JButton();
        jB_open = new javax.swing.JButton();
        jB_save = new javax.swing.JButton();
        jl_elapsedTime = new javax.swing.JLabel();
        jSlider1 = new javax.swing.JSlider();
        jLabel4 = new javax.swing.JLabel();
        jL_strengthValue = new javax.swing.JLabel();

        setMinimumSize(new java.awt.Dimension(685, 165));
        setPreferredSize(new java.awt.Dimension(685, 165));
        setLayout(null);

        jLabel1.setText("File :");
        jLabel1.setName("jl_file"); // NOI18N
        add(jLabel1);
        jLabel1.setBounds(10, 10, 80, 20);

        jtf_fileName.setName(""); // NOI18N
        add(jtf_fileName);
        jtf_fileName.setBounds(90, 10, 560, 20);

        jLabel2.setText("Password :");
        jLabel2.setName("jf_password"); // NOI18N
        add(jLabel2);
        jLabel2.setBounds(10, 40, 180, 20);

        jPasswordField1.setName(""); // NOI18N
        jPasswordField1.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                jPasswordField1FocusLost(evt);
            }
        });
        jPasswordField1.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                jPasswordField1KeyReleased(evt);
            }
        });
        add(jPasswordField1);
        jPasswordField1.setBounds(190, 40, 460, 20);

        jLabel3.setText("Repeat Password :");
        jLabel3.setName("jl_repeatPassword"); // NOI18N
        add(jLabel3);
        jLabel3.setBounds(10, 70, 180, 20);

        jPasswordField2.setName(""); // NOI18N
        jPasswordField2.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                jPasswordField2FocusLost(evt);
            }
        });
        jPasswordField2.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                jPasswordField2KeyReleased(evt);
            }
        });
        add(jPasswordField2);
        jPasswordField2.setBounds(190, 70, 460, 20);

        jButton1.setText("...");
        jButton1.setMinimumSize(new java.awt.Dimension(20, 20));
        jButton1.setName("jb_chooseFile"); // NOI18N
        jButton1.setPreferredSize(new java.awt.Dimension(20, 20));
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });
        add(jButton1);
        jButton1.setBounds(660, 10, 20, 20);

        jProgressBar1.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(255, 0, 0)));
        jProgressBar1.setName("jpb_progress"); // NOI18N
        jProgressBar1.setStringPainted(true);
        add(jProgressBar1);
        jProgressBar1.setBounds(340, 130, 330, 20);

        jb_cancel.setText("Cancel");
        jb_cancel.setName("jb_cancel"); // NOI18N
        jb_cancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jb_cancelActionPerformed(evt);
            }
        });
        add(jb_cancel);
        jb_cancel.setBounds(100, 130, 150, 23);

        jB_open.setText("Open");
        jB_open.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jB_openActionPerformed(evt);
            }
        });
        add(jB_open);
        jB_open.setBounds(420, 100, 120, 23);

        jB_save.setText("Save");
        jB_save.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jB_saveActionPerformed(evt);
            }
        });
        add(jB_save);
        jB_save.setBounds(550, 100, 120, 23);

        jl_elapsedTime.setMinimumSize(new java.awt.Dimension(100, 20));
        jl_elapsedTime.setName(""); // NOI18N
        jl_elapsedTime.setPreferredSize(new java.awt.Dimension(100, 20));
        add(jl_elapsedTime);
        jl_elapsedTime.setBounds(270, 130, 70, 20);

        jSlider1.setMaximum(256);
        jSlider1.setMinimum(1);
        jSlider1.setValue(256);
        jSlider1.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                jSlider1StateChanged(evt);
            }
        });
        add(jSlider1);
        jSlider1.setBounds(80, 100, 270, 26);

        jLabel4.setText("Strength :");
        add(jLabel4);
        jLabel4.setBounds(10, 100, 70, 14);

        jL_strengthValue.setText(" 256");
        jL_strengthValue.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));
        add(jL_strengthValue);
        jL_strengthValue.setBounds(360, 100, 30, 16);
    }// </editor-fold>//GEN-END:initComponents

    private void jPasswordField1FocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_jPasswordField1FocusLost
        // TODO add your handling code here:
        M_checkPasswords();
    }//GEN-LAST:event_jPasswordField1FocusLost

    private void jPasswordField1KeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_jPasswordField1KeyReleased
        // TODO add your handling code here:

        M_checkPasswords();
    }//GEN-LAST:event_jPasswordField1KeyReleased

    private void jPasswordField2FocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_jPasswordField2FocusLost
        // TODO add your handling code here:
        M_checkPasswords();
    }//GEN-LAST:event_jPasswordField2FocusLost

    private void jPasswordField2KeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_jPasswordField2KeyReleased
        // TODO add your handling code here:

        M_checkPasswords();
    }//GEN-LAST:event_jPasswordField2KeyReleased

    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        // TODO add your handling code here:
/*
        String path = ApplicationConfiguration.M_getInstance().
        M_getStrParamConfiguration(ApplicationConfiguration.CONF_LAST_DIRECTORY);

		JFileChooser chooser=new JFileChooser(path);
        FileNameExtensionFilter fnef =
        new FileNameExtensionFilter(	StringsConfiguration.M_getInstance().
            M_getStrParamConfiguration(StringsConfiguration.CONF_ENCRYPTED_FILE_JFE),
            "jfe" );
        chooser.addChoosableFileFilter(fnef);
        Utils.M_changeFontToApplicationFontSize_forComponent( chooser );

        int returnVal = chooser.showOpenDialog(this);
        if(returnVal == JFileChooser.APPROVE_OPTION)
        {
            path=chooser.getSelectedFile().getParentFile().getAbsolutePath();
            jtf_fileName.setText( chooser.getSelectedFile().toString() );

            ApplicationConfiguration.M_getInstance().M_setStrParamConfiguration(
                ApplicationConfiguration.CONF_LAST_DIRECTORY,
                ( path ) );
        }
*/
		String path = "";

		String fileName = _controller.textCrypt_chooseFile();
		setFileName( fileName );

    }//GEN-LAST:event_jButton1ActionPerformed

    private void jb_cancelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jb_cancelActionPerformed
        // TODO add your handling code here:

		if( _operationCancellation != null )
			_operationCancellation.M_setHasToCancel(true);

    }//GEN-LAST:event_jb_cancelActionPerformed

    private void jB_saveActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jB_saveActionPerformed
        // TODO add your handling code here:

		_controller.textCrypt_saveFile();

    }//GEN-LAST:event_jB_saveActionPerformed

    private void jB_openActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jB_openActionPerformed
        // TODO add your handling code here:

		_controller.textCrypt_openFile();

    }//GEN-LAST:event_jB_openActionPerformed

    private void jSlider1StateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_jSlider1StateChanged
        // TODO add your handling code here:

		jL_strengthValue.setText( " " + jSlider1.getValue() );

    }//GEN-LAST:event_jSlider1StateChanged


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jB_open;
    private javax.swing.JButton jB_save;
    private javax.swing.JButton jButton1;
    private javax.swing.JLabel jL_strengthValue;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JPasswordField jPasswordField1;
    private javax.swing.JPasswordField jPasswordField2;
    private javax.swing.JProgressBar jProgressBar1;
    private javax.swing.JSlider jSlider1;
    private javax.swing.JButton jb_cancel;
    private javax.swing.JLabel jl_elapsedTime;
    private javax.swing.JTextField jtf_fileName;
    // End of variables declaration//GEN-END:variables


	public boolean M_arePasswordsEqual()
	{
		return( StringFunctions.M_compare( jPasswordField1.getPassword(), jPasswordField2.getPassword() ) == 0 );
	}

	protected void M_checkPasswords()
	{
		if( ! M_arePasswordsEqual() )
		{
			M_showDifferentPasswords_RED();
		}
		else
		{
			M_showEqualPasswords_BLACK();
		}
	}
    
	protected void M_showDifferentPasswords_RED()
	{
		M_showColorPasswords(Color.RED);
	}

	protected void M_showEqualPasswords_BLACK()
	{
		M_showColorPasswords(Color.BLACK);
	}

	protected void M_showColorPasswords( Color color )
	{
		jPasswordField1.setForeground(color);
		jPasswordField2.setForeground(color);
	}

	public MapResizeRelocateComponentItem getResizeRelocateInfo()
	{
		if( _mapRRCI == null )
		{
			_mapRRCI = createResizeRelocateInfo();
		}

		return( _mapRRCI );
	}

	protected MapResizeRelocateComponentItem createResizeRelocateInfo()
	{
		MapResizeRelocateComponentItem mapRRCI = new MapResizeRelocateComponentItem();

		try
		{
			mapRRCI.putResizeRelocateComponentItem( jtf_fileName, ResizeRelocateItem.RESIZE_TO_RIGHT );
			mapRRCI.putResizeRelocateComponentItem( jButton1, ResizeRelocateItem.MOVE_TO_RIGHT );
		}
		catch( Throwable th )
		{
			th.printStackTrace();
		}
	
		return( mapRRCI );
	}

	protected void M_setProgressValue( int progress )
	{
		jProgressBar1.setValue(progress);
		Date now = new Date();
		String textForElapsedTime = String.valueOf( now.getTime() - a_beginningOfOperation.getTime() ) + " ms";
		jl_elapsedTime.setText( textForElapsedTime );
	}
	
	@Override
	public void up_childStarts()
	{
		a_beginningOfOperation = new Date();
		up_updateProgressFromChild(0);
	}

	@Override
	public void up_updateProgressFromChild(double completedFraction)
	{
		int completedPercentage = (int) ( completedFraction * 100 );
		M_setProgressValue( completedPercentage );
	}

	@Override
	public void up_childEnds()
	{
		M_setProgressValue( 100 );
	}

	@Override
	public OperationCancellation up_getOperationCancellation()
	{
		return( _operationCancellation );
	}

	@Override
	public void up_setDebug( boolean debug )
	{
	}

	public char[] getPassword()
	{
		return( jPasswordField1.getPassword() );
	}

	public void blockOpenAndSave()
	{
		jB_open.setEnabled( false );
		jB_save.setEnabled( false );
		jb_cancel.setEnabled( true );
	}

	public void unblockOpenAndSave()
	{
		jB_open.setEnabled( true );
		jB_save.setEnabled( true );
		jb_cancel.setEnabled( false );
	}
}
