/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/gpl-3.0.txt
 *
 */
package com.frojasg1.textcrypt.startapp;

import com.frojasg1.general.desktop.dialogs.implementation.DesktopDialogsWrapper;
import com.frojasg1.general.desktop.view.ToolTipLookAndFeel;
import com.frojasg1.general.desktop.workers.GenericSwingWorker;
import com.frojasg1.general.dialogs.DialogsWrapper;
import com.frojasg1.generic.workers.GenericWorkerResult;
import com.frojasg1.textcrypt.application.LicenseJDialog;
import com.frojasg1.textcrypt.application.TextCryptJFrame;
import com.frojasg1.textcrypt.application.configuration.ApplicationConfiguration;
import java.awt.Component;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.JDialog;

/**
 *
 * @author Usuario
 */
public class StartApplication
{
	protected static final int WATING_FOR_IMPORT_CONFIGURATION = 1;
	protected static final int WATING_FOR_CHECK_LICENSES = 2;
	protected static final int WATING_FOR_ACCEPT_LICENSES = 3;
	protected static final int WATING_FOR_CREATE_MAIN_WINDOW = 4;
	protected static final int WATING_FOR_SHOW_MAIN_WINDOW = 5;
	
	protected boolean _exceptionThrown = false;

//	protected Splash _splash = null;

	protected OpenConfiguration _oc = null;

	protected int _totalNumberOfStepsToStart = -1;
	protected int _numberOfStepsCompleted = 0;

	protected TextCryptJFrame _mainWindow = null;

	public StartApplication()
	{
	}

	public void startApplication( )
	{
		try
		{
			javax.swing.UIManager.setLookAndFeel( new ToolTipLookAndFeel() );
		} catch (javax.swing.UnsupportedLookAndFeelException ex) {
			java.util.logging.Logger.getLogger(TextCryptJFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		}

		openConfigurationAndShowSplash();
	}

	protected void openConfigurationAndShowSplash()
	{
		try
		{
			_oc = new OpenConfiguration();
//			_totalNumberOfStepsToStart = getTotalNumberOfSteps();	// it needs _oc and ApplicationConfiguration to be created previously.

//			_splash = Splash.instance();
//			_splash.setCurrentActivityFromLabel( Splash.CONF_OPENING_CONFIGURATION );
//			_oc.setParent( _splash );
/*
			java.awt.EventQueue.invokeLater(new Runnable()  {
				public void run() {
					try
					{
						_splash.setVisible( true );
					}
					catch( Throwable th )
					{
						th.printStackTrace();
					}
				}
			});

			incrementStepsCompleted();
*/
			GenericWaitWorker gww = new GenericWaitWorker( 500, WATING_FOR_IMPORT_CONFIGURATION );
			gww.execute();
		}
		catch( Throwable th )
		{
			th.printStackTrace();
			System.exit(1);
		}
	}

	protected void importConfiguration()
	{
		if( _exceptionThrown )
			System.exit(1);

		try
		{
			if( _oc.importConfigurationIfNecessary() )
			{
				this.incrementStepsCompleted();
			}

			GenericWaitWorker gww = new GenericWaitWorker( 150, WATING_FOR_CHECK_LICENSES );
			gww.execute();
		}
		catch( Throwable th )
		{
			th.printStackTrace();
			System.exit(1);
		}
	}

	protected void checkLicenses()
	{
		if( _exceptionThrown )
			System.exit(1);

		try
		{
			if( !ApplicationConfiguration.instance().getLicensesHaveBeenAccepted() )
			{
//				_splash.setCurrentActivityFromLabel( Splash.CONF_SHOWING_LICENSES );

				GenericWaitWorker gww = new GenericWaitWorker( 100, WATING_FOR_ACCEPT_LICENSES );
				gww.execute();
			}
			else
			{
//				_splash.setCurrentActivityFromLabel( Splash.CONF_CREATING_MAIN_WINDOW );

				GenericWaitWorker gww = new GenericWaitWorker( 100, WATING_FOR_CREATE_MAIN_WINDOW );
				gww.execute();
			}
		}
		catch( Throwable th )
		{
			th.printStackTrace();
			System.exit(1);
		}
	}

	protected void acceptLicenses()
	{
		if( _exceptionThrown )
			System.exit(1);

		try
		{
			LicenseJDialog licenseJDialog = new LicenseJDialog( (JDialog)null, ApplicationConfiguration.instance() );
			licenseJDialog.setVisible( true );

			if( !licenseJDialog.getLicensesHaveBeenAcceptedAndWeHaveToStart() )
			{
				_oc.eraseConfiguration();
				System.exit(1);
			}

			ApplicationConfiguration.instance().M_saveConfiguration();
			licenseJDialog.closeAndReleaseWindow();

			incrementStepsCompleted();

//			_splash.setCurrentActivityFromLabel( Splash.CONF_CREATING_MAIN_WINDOW );

			GenericWaitWorker gww = new GenericWaitWorker( 100, WATING_FOR_CREATE_MAIN_WINDOW );
			gww.execute();
		}
		catch( Throwable th )
		{
			th.printStackTrace();
			System.exit(1);
		}
	}

	protected void createMainWindow()
	{
		if( _exceptionThrown )
			System.exit(1);

		try
		{
			_mainWindow = new TextCryptJFrame( );
			incrementStepsCompleted();

			GenericWaitWorker gww = new GenericWaitWorker( 500, WATING_FOR_SHOW_MAIN_WINDOW );
			gww.execute();
		}
		catch( Throwable th )
		{
			th.printStackTrace();
//			DesktopDialogsWrapper.instance().showMessageDialog(null,
//						_splash.getInternationalString( _splash.CONF_PROBLEM_LAUNCHING_THE_APPLICATION ),
//						"ERROR", DialogsWrapper.ERROR_MESSAGE );
			DesktopDialogsWrapper.instance().showMessageDialog(null,
						"Problem launching the application",
						"ERROR", DialogsWrapper.ERROR_MESSAGE );

			System.exit(1);
		}
	}

	protected void showMainWindow()
	{
		if( _exceptionThrown )
			System.exit(1);

		try
		{
			_mainWindow.setVisible(true);

//			Splash.free();
		}
		catch( Throwable th )
		{
			th.printStackTrace();
			System.exit(1);
		}
	}

	protected int getTotalNumberOfSteps()
	{
		int totalNumberOfSteps = 2;

		if( _oc.getHasToImport() )
			totalNumberOfSteps++;

		if( !ApplicationConfiguration.instance().getLicensesHaveBeenAccepted() )
			totalNumberOfSteps++;

		return( totalNumberOfSteps );
	}

	protected int getProgress( int numberOfCompletedSteps )
	{
		return( ( 100 * numberOfCompletedSteps ) / _totalNumberOfStepsToStart );
	}

	protected void incrementStepsCompleted()
	{
//		_splash.setProgress( getProgress( ++_numberOfStepsCompleted ) );		
	}

	protected abstract class StartAppBaseWorker extends GenericSwingWorker implements PropertyChangeListener
	{
		public static final String NEW_ACTION_LABEL_PROPERTY_NAME = "newactionlabel";
		public static final String NEW_ACTION_STRING_PROPERTY_NAME = "newactionstring";
		public static final String NEW_STEP_COMPLETED_PROPERTY_NAME = "newstepcompleted";

		protected StartAppBaseWorker( Component parent )
		{
			super( parent );
			addPropertyChangeListener(this);
		}

		public void propertyChange(PropertyChangeEvent evt)
		{
			if (NEW_ACTION_LABEL_PROPERTY_NAME.equals( evt.getPropertyName() ) )
			{
//				_splash.setCurrentActivityFromLabel( (String) evt.getNewValue() );
			}
			else if( NEW_ACTION_STRING_PROPERTY_NAME.equals( evt.getPropertyName() ) )
			{
//				_splash.setCurrentActivity( (String) evt.getNewValue() );
			}
			else if( NEW_STEP_COMPLETED_PROPERTY_NAME.equals( evt.getPropertyName() ) )
			{
				incrementStepsCompleted();
			}
		}

		public void newStepCompleted()
		{
			firePropertyChange(NEW_STEP_COMPLETED_PROPERTY_NAME, null, null);
		}

		public void setCurrentActivityFromLabel( String label )
		{
			firePropertyChange(NEW_ACTION_LABEL_PROPERTY_NAME, null, label);
		}

		public void setCurrentActivity( String currentActivityString )
		{
			firePropertyChange(NEW_ACTION_STRING_PROPERTY_NAME, null, currentActivityString);
		}
	}

	protected class GenericWaitWorker extends StartAppBaseWorker
	{
		int _millisecondsToWait = -1;
		int _nextStep = -1;
		
		protected GenericWaitWorker( int millisecondsToWait, int nextStep )
		{
			super( null );

			_millisecondsToWait = millisecondsToWait;
			_nextStep = nextStep;
		}

		@Override
		protected GenericWorkerResult doInBackground() throws Exception
		{
			try
			{
				Thread.sleep( _millisecondsToWait );
			}
			catch( Throwable th )
			{
				th.printStackTrace();
			}

			return( null );
		}

		@Override
		protected void done()
		{
			switch( _nextStep )
			{
				case WATING_FOR_IMPORT_CONFIGURATION:	importConfiguration();		break;
				case WATING_FOR_CHECK_LICENSES:			checkLicenses();			break;
				case WATING_FOR_ACCEPT_LICENSES:		acceptLicenses();			break;
				case WATING_FOR_CREATE_MAIN_WINDOW:		createMainWindow();			break;
				case WATING_FOR_SHOW_MAIN_WINDOW:		showMainWindow();			break;
			}
		}
	}
	
}
