/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general;

import com.frojasg1.general.desktop.files.charset.CharsetManager;
import java.awt.image.BufferedImage;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import javax.imageio.ImageIO;
import javax.swing.text.DefaultStyledDocument;
import javax.swing.text.Document;
import javax.swing.text.rtf.RTFEditorKit;

/**
 *
 * @author Usuario
 */
public class ResourceFunctions
{
	public static InputStream getInputStreamOfResource( String resourcePath ) throws IOException
	{
		InputStream in = null;
		try
		{
			ClassLoader loader = ClassLoader.getSystemClassLoader ();
			in = loader.getResourceAsStream (resourcePath);

			if( in == null )
			{
				try
				{
					in = Class.forName("ResourceFunctions").getClassLoader().getResource(resourcePath).openStream();
				}
				catch( Throwable th )
				{
					in = null;
				}
			}
		}
		catch( Throwable th )
		{
			th.printStackTrace();
			
			try
			{
				if( in != null )
					in.close();
			}
			catch( Throwable th2 )
			{
				
			}
		}
		
		return( in );
	}
	
	public static BufferedImage loadResourceImage( String resourcePath )
	{
		BufferedImage result = null;

		InputStream in = null;
		try
		{
			in = getInputStreamOfResource( resourcePath );
			result = ImageIO.read(in);
		}
		catch( Throwable th )
		{
			th.printStackTrace();
			result = null;
		}
		finally
		{
			try
			{
				if( in != null )
					in.close();
			}
			catch( Throwable th )
			{
				
			}
		}
		return( result );
	}

	public static Document loadResourceRTFDocument( String resourcePath )
	{
		Document result = null;

		InputStream is = null;
		InputStreamReader isr = null;
		try
		{
			is = getInputStreamOfResource( resourcePath );
			if( is != null )
			{
//				is.mark( 2000000000 );
				String charsetName = CharsetManager.instance().M_detectCharset(is );

				try
				{
					is.close();
				}
				catch( Throwable th )
				{
				}

				is = getInputStreamOfResource( resourcePath );

//				is.reset();		// does not work
				if( charsetName != null )	isr = new InputStreamReader( is, charsetName );
				else							isr = new InputStreamReader( is );


				RTFEditorKit ek = new RTFEditorKit();
				Document doc = new DefaultStyledDocument();
				ek.read( isr, doc, 0);

				result = doc;
			}
		}
		catch( Throwable th )
		{
			th.printStackTrace();
			result = null;
		}
		finally
		{
			try
			{
				if( isr != null )
					isr.close();
				if( is != null )
					is.close();
			}
			catch( Throwable th )
			{
			}
		}
		return( result );
	}

	public static void copyBinaryResourceToFile( String longResourceName,
												String longFileName )
	{
		InputStream is = null;
		FileOutputStream fos = null;

		try
		{
			is = ResourceFunctions.getInputStreamOfResource( longResourceName );
			fos = new FileOutputStream(longFileName);

			byte[] buffer = new byte[1024];
			int noOfBytes = 0;

			// read bytes from source file and write to destination file
			while ((noOfBytes = is.read(buffer)) != -1)
			{
				fos.write(buffer, 0, noOfBytes);
			}
		}
		catch (Throwable th)
		{
			th.printStackTrace();
		}
		finally
		{
			// close the streams using close method
			try
			{
				if (is != null)
				{
					is.close();
				}
				if (fos != null)
				{
					fos.close();
				}
			}
			catch (IOException ioe)
			{
				ioe.printStackTrace();
				System.out.println("Error while closing stream: " + ioe);
			}
		}
	}

}
