/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 *
 * @author Usuario
 */
public class StringFunctions
{
	public static final String RETURN = System.getProperty("line.separator");


	public static int M_compare( char[] array1, char[] array2 )
	{
		int result = 0;
		
		if( ( array1 == null ) && ( array2 == null ) )
			result = 0;
		else if( array1 == null )
			result = -1;
		else if( array2 == null )
			result = 1;
		else
		{
			int length = ( array1.length>array2.length ? array2.length : array1.length );

			for( int ii=0; (ii<length) && (result==0); ii++ )
			{
				if( array1[ii] > array2[ii] )	result = 1;
				else if( array1[ii] < array2[ii] )	result = -1;
			}

			if( (result == 0) && ( array1.length != array2.length ) )
			{
				if( array1.length > array2.length )	result = 1;
				else	result = -1;
			}
		}

		return( result );
	}

	public static byte[] M_getBytes( char[] array )
	{
		byte[] result = null;
		
		if( array != null )
		{
			result = new byte[ array.length*4 ];
			for( int ii=0; ii<array.length; ii++ )
			{
				byte[] intBuffer = HexadecimalFunctions.M_getBufferFromInteger( array[ii] );
				System.arraycopy(intBuffer, 0, result, ii*4, 4 );
			}
		}

		return( result );
	}

	public static String[] runRegEx( String patronRegEx, String str )
	{
		String[] result = null;
		Pattern pat = Pattern.compile( patronRegEx );
		Matcher mat = pat.matcher( str );
		if( mat.matches() )
		{
			int numGrupos = mat.groupCount();
			result = new String[ numGrupos + 1 ];
			for( int ii=0; ii<numGrupos+1; ii++ )
			{
				result[ ii ] = mat.group( ii );
			}
		}
		return( result );
	}
	
	public static void append( StringBuilder sb, String str )
	{
		if( str != null ) sb.append( str );
	}
	
	public static int indexOfAnyChar( String string, String chars, int pos )
	{
		int ii = pos;
		if( ii<0 ) ii=0;

		int posFound = -1;
		for( ; (ii< string.length()) && (posFound == -1 ); ii++ )
		{
			if( chars.indexOf( string.substring( ii, ii+1 ) ) > -1 )
				posFound = ii;
		}

		return( posFound );
	}

	public static int lastIndexOfAnyChar( String string, String chars, int pos )
	{
		int ii = pos;
		if( ii>(string.length()-1) ) ii=string.length()-1;

		int posFound = -1;
		for( ; ( ii >= 0 ) && ( posFound == -1 ); ii-- )
		{
			if( chars.indexOf( string.substring( ii, ii+1 ) ) > -1 )
				posFound = ii;
		}

		return( posFound );
	}

	public static int indexOfAnyChar( StringBuilder string, String charsToFind, int pos )
	{
		int ii = pos;
		if( ii<0 ) ii=0;

		int posFound = -1;
		for( ; (ii< string.length()) && (posFound == -1 ); ii++ )
		{
			char cc = string.charAt(ii);
			for( int jj=0; (jj<charsToFind.length()) && (posFound == -1); jj++ )
			{
				if( charsToFind.charAt(jj) == cc )
					posFound = ii;
			}
		}

		return( posFound );
	}

	public static int lastIndexOfAnyChar( StringBuilder string, String charsToFind, int pos )
	{
		int ii = pos;
		if( ii>(string.length()-1) ) ii=string.length()-1;

		int posFound = -1;
		for( ; ( ii >= 0 ) && ( posFound == -1 ); ii-- )
		{
			char cc = string.charAt(ii);
			for( int jj=0; (jj<charsToFind.length()) && (posFound == -1); jj++ )
			{
				if( charsToFind.charAt(jj) == cc )
					posFound = ii;
			}
		}

		return( posFound );
	}

	/**
	 * Function which makes a translation of characters.
	 * originSetOfChars must have the same length than destinationSetOfChars.
	 * The first character of originSetOfChars is replaced by the first character
	 * of destinationSetOfChars, an so on.
	 * 
	 * @param string					String to transform
	 * @param originSetOfChars			Set of chars to find in the string.
	 * @param destinationSetOfChars		Set of chars to transform with.
	 * @return							Transformed string
	 */
	public static String replaceSetOfChars( String string, String originSetOfChars, String destinationSetOfChars )
	{
		StringBuilder sb = new StringBuilder();

		if( string != null )
		{
			if( ( originSetOfChars != null ) &&
				( destinationSetOfChars != null ) &&
				( originSetOfChars.length() == destinationSetOfChars.length() )
			   )
			{
				for( int ii=0; ii<string.length(); ii++ )
				{
					char newChar = string.charAt( ii );
					int index = originSetOfChars.indexOf( newChar );
					if( index != -1 )
						newChar = destinationSetOfChars.charAt( index );

					sb.append( newChar );
				}
			}
			else
				sb.append( string );
		}

		return( sb.toString() );
	}
	
	public static String removeAllCharacters( String inputString, String charToRemove )
	{
		StringBuilder sb = new StringBuilder();

		if( inputString != null )
		{
			if( charToRemove != null )
			{
				for( int ii=0; ii<inputString.length(); ii++ )
				{
					char newChar = inputString.charAt( ii );
					int index = charToRemove.indexOf( newChar );
					if( index == -1 )
						sb.append( newChar );
				}
			}
			else
				sb.append( inputString );
		}

		return( sb.toString() );
	}
	
	/**
	 * 
	 * @param str		String from which to take the char
	 * @param position	position of the char to be extracted
	 * @return			returns the char of the position (if the string is long enough) and the char does not repeat towards the end of the string.
	 *					if not, the functions returns null
	 */
	public static String getNotRepeatedChar( String str, int position )
	{
		String result = null;
		
		if( ( str != null ) && ( str.length() > position ) )
		{
			result = str.substring(position, position + 1);
			boolean repeated = false;
			if( str.length() > (position+1) )
			{
				int pos = str.indexOf( result, position + 1 );
				if( ( pos > 0 ) && ( pos < str.length() ) )
				{
					repeated = true;
				}
				if( repeated )
					result = null;
			}
		}

		return( result );
	}
	
	public static String buildStringFromRepeatedChar( Character ch, int numberOfRepetitions )
	{
		String result = null;
		
		if( ch != null )
		{
			StringBuilder sb = new StringBuilder( numberOfRepetitions );
			for( int ii=0; ii<numberOfRepetitions; ii++ )
				sb.append( ch );
			
			result = sb.toString();
		}
		
		return( result );
	}
}
