/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.hotmail.frojasg1.applications.common.components.internationalization.window;

import com.hotmail.frojasg1.applications.common.components.internationalization.InternException;
import com.hotmail.frojasg1.applications.common.components.internationalization.JFrameInternationalization;
import com.hotmail.frojasg1.applications.common.components.resizecomp.MapResizeRelocateComponentItem;
import com.frojasg1.applications.common.configuration.application.BaseApplicationConfigurationInterface;
import com.frojasg1.applications.common.configuration.application.ChangeLanguageClientInterface;
import com.frojasg1.applications.common.configuration.application.ChangeLanguageServerInterface;
import com.frojasg1.general.ArrayFunctions;
import com.frojasg1.general.desktop.ImageFunctions;
import com.frojasg1.general.ResourceFunctions;
import com.frojasg1.general.desktop.dialogs.DesktopViewComponent;
import com.frojasg1.general.exceptions.ConfigurationException;
import com.frojasg1.general.desktop.mouse.AvailableCursors;
import com.frojasg1.general.dialogs.ViewComponent;
import com.frojasg1.generic.GenericFunctions;
import com.frojasg1.generic.languages.ListOfLanguagesResult_int;
import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.image.BufferedImage;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;
import javax.swing.AbstractButton;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JMenu;
import javax.swing.JPopupMenu;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.MenuElement;

/**
 *
 * @author Usuario
 */
public class InternationalizedJFrame extends javax.swing.JFrame
										implements InternationalizedWindow,
													ChangeLanguageClientInterface,
													ChangeLanguageServerInterface,
													ItemListener
{

	protected JFrameInternationalization a_intern = null;

	protected BaseApplicationConfigurationInterface _appliConf = null;

	protected boolean _previousValueOfHightlightFocus = false;
	protected boolean _alwaysHighlightFocus = false;
	
	protected Component _lastFocusedComponentDrawn = null;

	protected Component _highlightFocusComponent = null;

	protected Map<Component, Cursor> _rollbackWaitCursorMap = null;

	protected Object _mutexForPaint = new Object();

	protected BufferedImage _overlappedImage = null;
	protected Point _overlappedImageLocation = null;

	protected Component _componentToHighLight = null;

	protected boolean _hasToClearMarkedComponent = false;

	protected ButtonGroup _languageRadiobuttonGroup = null;
	protected JPopupMenu _languageSubmenu = null;

	protected boolean _initializing = true;

//	protected int _debugCounter = 0;
	
	public InternationalizedJFrame( BaseApplicationConfigurationInterface applicationConfiguration )
	{
		initComponents();

		setAppliConf( applicationConfiguration );
	}

	@Override
	public final void setAppliConf( BaseApplicationConfigurationInterface applicationConfiguration )
	{
		unregisterFromChangeLanguageAsObserver();

		_appliConf = applicationConfiguration;

		if( getAppliConf() != null )
			getAppliConf().registerChangeLanguageObserver( this );

		String resourceForIcon = _appliConf.getResourceNameForApplicationIcon();
		if( resourceForIcon != null )
			setIcon( resourceForIcon );
	}
	
	protected void createInternationalization(	String mainFolder,
												String applicationName,
												String group,
												String paquetePropertiesIdiomas,
												String configurationBaseFileName,
												Component parentFrame,
												Component parentParentFrame,
												Vector<JPopupMenu> vPUMenus,
												boolean hasToPutWindowPosition,
												MapResizeRelocateComponentItem mapRRCI
											)
	{
/*
		MapResizeRelocateComponentItem mapRRCI = new MapResizeRelocateComponentItem();
		try
		{
			mapRRCI.putResizeRelocateComponentItem( jPanel1, ResizeRelocateItem.RESIZE_TO_RIGHT + ResizeRelocateItem.RESIZE_TO_BOTTOM );
			mapRRCI.putResizeRelocateComponentItem( jSPmainSplit, ResizeRelocateItem.RESIZE_TO_RIGHT + ResizeRelocateItem.RESIZE_TO_BOTTOM );
			mapRRCI.putResizeRelocateComponentItem( jPanelNavigatorContainer1, ResizeRelocateItem.RESIZE_TO_RIGHT + ResizeRelocateItem.RESIZE_TO_BOTTOM );
			mapRRCI.putResizeRelocateComponentItem( jSPnavigatorSplit, ResizeRelocateItem.RESIZE_TO_RIGHT + ResizeRelocateItem.RESIZE_TO_BOTTOM );
			mapRRCI.putResizeRelocateComponentItem( jPanel4, ResizeRelocateItem.RESIZE_TO_RIGHT );
			mapRRCI.putResizeRelocateComponentItem( jPanel3, ResizeRelocateItem.RESIZE_TO_RIGHT );
			mapRRCI.putResizeRelocateComponentItem( _navigatorPanel, ResizeRelocateItem.RESIZE_TO_RIGHT + ResizeRelocateItem.RESIZE_TO_BOTTOM );

			mapRRCI.putResizeRelocateComponentItem( jScrollPane2, ResizeRelocateItem.RESIZE_TO_RIGHT + ResizeRelocateItem.RESIZE_TO_BOTTOM );
			mapRRCI.putResizeRelocateComponentItem( jTextPane1, ResizeRelocateItem.RESIZE_TO_RIGHT + ResizeRelocateItem.RESIZE_TO_BOTTOM );
			mapRRCI.putResizeRelocateComponentItem( jScrollPane1, ResizeRelocateItem.RESIZE_TO_RIGHT + ResizeRelocateItem.RESIZE_TO_BOTTOM );
			mapRRCI.putResizeRelocateComponentItem( jTListOfGames, ResizeRelocateItem.RESIZE_TO_RIGHT + ResizeRelocateItem.RESIZE_TO_BOTTOM );

			mapRRCI.putResizeRelocateComponentItem( jPanelChessBoardContainer, ResizeRelocateItem.RESIZE_TO_RIGHT + ResizeRelocateItem.RESIZE_TO_BOTTOM );
			mapRRCI.putResizeRelocateComponentItem( _chessBoardPanel, ResizeRelocateItem.RESIZE_TO_RIGHT + ResizeRelocateItem.RESIZE_TO_BOTTOM );
		}
		catch( Throwable th )
		{
			th.printStackTrace();
		}
*/
/*
		a_intern = new JFrameInternationalization(	ApplicationConfiguration.sa_MAIN_FOLDER,
													ApplicationConfiguration.sa_APPLICATION_NAME,
													ApplicationConfiguration.sa_CONFIGURATION_GROUP,
													ApplicationConfiguration.sa_PATH_PROPERTIES_IN_JAR,
													a_configurationBaseFileName,
													this,
													parent,
													a_vectorJpopupMenus,
													false,
													mapRRCI );
*/
		a_intern = new JFrameInternationalization();
		a_intern.initialize(	mainFolder,
								applicationName,
								group,
								paquetePropertiesIdiomas,
								configurationBaseFileName,
								parentFrame,
								parentParentFrame,
								vPUMenus,
								hasToPutWindowPosition,
								mapRRCI );

		try
		{
			a_intern.changeLanguage( getAppliConf().getLanguage() );
		}
		catch( Throwable th )
		{
			th.printStackTrace();
		}
		
		try
		{
			changeFontSize( getAppliConf().getFontSizeFactor() );
		}
		catch( Throwable th )
		{
			th.printStackTrace();
		}
	}

	@Override
	public Object getMutexForPaint()
	{
		return( _mutexForPaint );
	}
	
	protected void setIcon( String resourceToImage )
	{
//		String resourceToImage = "com/frojasg1/chesspdfbrowser/resources/icons/App.icon.png";
		BufferedImage image = ResourceFunctions.loadResourceImage(resourceToImage);
		if( image != null ) setIconImage( image );
	}

	/**
	 * Creates new form IntenationalizationJFrame
	 */
	public InternationalizedJFrame() {
//		initComponents();
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	@SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                formWindowClosing(evt);
            }
        });
        addComponentListener(new java.awt.event.ComponentAdapter() {
            public void componentResized(java.awt.event.ComponentEvent evt) {
                formComponentResized(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 400, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 300, Short.MAX_VALUE)
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void formComponentResized(java.awt.event.ComponentEvent evt) {//GEN-FIRST:event_formComponentResized
        // TODO add your handling code here:

		if( a_intern != null )
		{
			try
			{
//				a_intern.saveGeneralConfiguration();
				a_intern.resizeOrRelocateComponent();
			}
			catch( Throwable th )
			{
				th.printStackTrace();
			}
		}

    }//GEN-LAST:event_formComponentResized

    private void formWindowClosing(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosing
        // TODO add your handling code here:

		formWindowClosingEvent();
    }//GEN-LAST:event_formWindowClosing

	/**
	 * @param args the command line arguments
	 */
	public static void main(String args[]) {
		/* Set the Nimbus look and feel */
        //<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /* If Nimbus (introduced in Java SE 6) is not available, stay with the default look and feel.
		 * For details see http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html 
		 */
		try {
			for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
				if ("Nimbus".equals(info.getName())) {
					javax.swing.UIManager.setLookAndFeel(info.getClassName());
					break;
				}
			}
		} catch (ClassNotFoundException ex) {
			java.util.logging.Logger.getLogger(InternationalizedJFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (InstantiationException ex) {
			java.util.logging.Logger.getLogger(InternationalizedJFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (IllegalAccessException ex) {
			java.util.logging.Logger.getLogger(InternationalizedJFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (javax.swing.UnsupportedLookAndFeelException ex) {
			java.util.logging.Logger.getLogger(InternationalizedJFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		}
        //</editor-fold>
        //</editor-fold>
        //</editor-fold>
        //</editor-fold>

		/* Create and display the form */
		java.awt.EventQueue.invokeLater(new Runnable() {
			public void run() {
				new InternationalizedJFrame().setVisible(true);
			}
		});
	}

    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables


	@Override
	public void unregisterFromChangeLanguageAsObserver()
	{
		if( getAppliConf() != null )
			getAppliConf().unregisterChangeLanguageObserver(this);
	}
	
	@Override
	public void releaseResources()
	{
		unregisterFromChangeLanguageAsObserver();

		a_intern=null;	// for the garbage collector to free the memory of the internationallization object and after the memory of this form
	}
	
	@Override
	public void changeLanguage( String language ) throws ConfigurationException, InternException
	{
		if( a_intern != null )
		{
//			a_intern.saveLanguageConfiguration();
			a_intern.changeLanguage( language );
		}
	}

	@Override
	public String getLanguage()
	{
		String result = ( (a_intern!=null) ? a_intern.M_getLanguage() : null );
		return( result );
	}

	@Override
	public void changeFontSize( float factor )
	{
		if( a_intern != null )
		{
			a_intern.M_changeFontSize(factor);
		}
	}

	@Override
	public void applyConfiguration() throws ConfigurationException, InternException
	{
		if( getAppliConf() != null )
			changeLanguage( getAppliConf().getLanguage() );
	}

/*
	protected void setConfigurationChanges()
	{
		
	}
*/

	@Override
	public void formWindowClosing( boolean closeWindow )
	{
/*
		setConfigurationChanges();

		if( _appliConf != null )
		{
			try
			{
				_appliConf.M_saveConfiguration();
			}
			catch( Throwable th )
			{
				th.printStackTrace();
			}
		}

		if( _stringsConf != null )
		{
			try
			{
				_stringsConf.M_saveConfiguration();
			}
			catch( Throwable th )
			{
				th.printStackTrace();
			}
		}
*/
		if( a_intern != null )
		{
			try
			{
				a_intern.saveConfiguration();
			}
			catch( Throwable th )
			{
				th.printStackTrace();
			}
		}

		if( closeWindow )
		{
			setVisible(false);
			releaseResources();
			dispose();
		}
	}

	@Override
	public BaseApplicationConfigurationInterface getAppliConf()
	{
		return( _appliConf );
	}
	
	@Override
	public void setSize( int width, int height )
	{
		super.setSize( width, height );
		
		if( a_intern != null )
			a_intern.resizeOrRelocateComponent();
	}

	@Override
	public void setSize( Dimension dim )
	{
		super.setSize( dim );
		
		if( a_intern != null )
			a_intern.resizeOrRelocateComponent();
	}

	@Override
	public void setBounds( int xx, int yy, int width, int height )
	{
		super.setBounds( xx, yy, width, height );
		
		if( a_intern != null )
			a_intern.resizeOrRelocateComponent();
	}

	@Override
	public void setBounds( Rectangle rect )
	{
		super.setBounds( rect );

		if( a_intern != null )
			a_intern.resizeOrRelocateComponent();
	}

	@Override
	public boolean getAlwaysHighlightFocus()
	{
		return( _alwaysHighlightFocus );
	}

	@Override
	public void setAlwaysHighlightFocus( boolean value )
	{
		_alwaysHighlightFocus = value;
		repaint();
	}

	@Override
	public void highlightComponent( ViewComponent vc )
	{
		if( vc instanceof DesktopViewComponent )
		{
			DesktopViewComponent dvc = (DesktopViewComponent) vc;
			_componentToHighLight = dvc.getComponent();

			repaint();
		}
	}

	@Override
	public void highlightFocus()
	{
		_previousValueOfHightlightFocus = _alwaysHighlightFocus;

		_highlightFocusComponent = JFrameInternationalization.getFocusedComponent();
		_alwaysHighlightFocus = true;

		repaint();
	}

	protected boolean markComponent( Graphics gc, Component comp )
	{
		boolean result = false;

		if( ( comp != null ) &&
			getRootPane().isAncestorOf(comp) )
		{
			int thick = 3;
			int gap = 2;

			Dimension size = comp.getSize();

			int width = (int) size.getWidth() + gap * 2;
			int height = (int) size.getHeight() + gap * 2;

			Point leftUpperCorner = comp.getLocationOnScreen();
			Point windowLeftUpperCorner = getLocationOnScreen();

			int xx = (int) ( leftUpperCorner.getX() - windowLeftUpperCorner.getX() - gap );
			int yy = (int) ( leftUpperCorner.getY() - windowLeftUpperCorner.getY() - gap );

			ImageFunctions.drawRect( gc, xx, yy, width, height, Color.RED, thick );

			result = true;
		}
		return( result );
	}

	protected boolean markFocusedComponent( Graphics gc )
	{
		boolean result = false;

		Component focusedComponent = JFrameInternationalization.getFocusedComponent();
		if( !(focusedComponent instanceof JButton) )
			result = markComponent( gc, focusedComponent );

		_lastFocusedComponentDrawn = focusedComponent;

		return( result );
	}

	@Override
	public Component getLastFocusedComponentDrawn()
	{
		return( _lastFocusedComponentDrawn );
	}

	protected boolean doHighlightFocus( Graphics gc )
	{
		boolean result = false;

		if( _alwaysHighlightFocus )
		{
			Component focusedComponent = JFrameInternationalization.getFocusedComponent();
			if( ( _highlightFocusComponent != null ) && (focusedComponent != _highlightFocusComponent ) )
			{
				_highlightFocusComponent = null;
				_alwaysHighlightFocus = _previousValueOfHightlightFocus;
			}

			if( _alwaysHighlightFocus )
				result = markFocusedComponent( gc );
		}

		return( result );
	}

	@Override
	public void paint( Graphics gc )
	{
		synchronized( _mutexForPaint )
		{
			super.paint( gc );

//		System.out.println( "Painting window : " + ++_debugCounter );

			boolean componentHasBeenMarked = false;
			
			if( _componentToHighLight != null )
			{
				componentHasBeenMarked = markComponent( gc, _componentToHighLight );
				_componentToHighLight = null;
			}

			if( ! componentHasBeenMarked )
				componentHasBeenMarked = doHighlightFocus( gc );

			_hasToClearMarkedComponent = componentHasBeenMarked;
			
			if( ( _overlappedImage != null ) && (_overlappedImageLocation != null ) )
			{
				ImageFunctions.paintClippedImage( this, gc, _overlappedImage, _overlappedImageLocation );
			}
		}
	}

	@Override
	public void changeToWaitCursor()
	{
		if( a_intern != null )
			_rollbackWaitCursorMap = a_intern.M_changeCursor( AvailableCursors._waitCursor );
	}

	@Override
	public void revertChangeToWaitCursor()
	{
		if( a_intern != null )
		{
			a_intern.M_rollbackChangeCursor( _rollbackWaitCursorMap );
			_rollbackWaitCursorMap = null;
		}
	}

	@Override
	public void setOverlappedImage( BufferedImage image, Point position )
	{
		synchronized( _mutexForPaint )
		{
			_overlappedImage = image;
			_overlappedImageLocation = position;
			repaint();
		}
	}

	@Override
	public int getWidth_forOverlappingImage()
	{
		return( getWidth() );
	}

	@Override
	public int getHeight_forOverlappingImage()
	{
		return( getHeight() );
	}
	
	@Override
	public Point getLocationOnScreen_forOverlappingImage()
	{
		return( getLocationOnScreen() );
	}

	@Override
	public void registerToChangeLanguageAsObserver(ChangeLanguageServerInterface conf)
	{
		throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
	}

	public void registerInternationalString( String label, String value )
	{
		a_intern.registerInternationalString(label, value);
	}

	public String getInternationalString( String label )
	{
		String result = null;

		if( a_intern != null )
			result = a_intern.getInternationalString(label);

		return( result );
	}

	@Override
	public void setLanguage(String language)
	{
		throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
	}

	@Override
	public void registerChangeLanguageObserver(ChangeLanguageClientInterface requestor)
	{
		if( a_intern != null )
		{
			a_intern.getLanguageConfiguration().registerChangeLanguageObserver( requestor );
		}
	}

	@Override
	public void unregisterChangeLanguageObserver(ChangeLanguageClientInterface requestor)
	{
		if( a_intern != null )
		{
			a_intern.getLanguageConfiguration().unregisterChangeLanguageObserver( requestor );
		}
	}

	@Override
	public void activateChangeLanguageNotifications(boolean value)
	{
		if( a_intern != null )
		{
			a_intern.getLanguageConfiguration().activateChangeLanguageNotifications( value );
		}
	}

	@Override
	public boolean areChangeLanguageNotificationsActivated()
	{
		boolean result = false;

		if( a_intern != null )
		{
			result = a_intern.getLanguageConfiguration().areChangeLanguageNotificationsActivated();
		}

		return( result );
	}

	@Override
	public void formWindowClosingEvent( )
	{
		formWindowClosing( true );
	}

	@Override
	public Component getComponent()
	{
		return( this );
	}

	@Override
	public boolean hasToClearMarkedComponent()
	{
		return( _hasToClearMarkedComponent );
	}

	@Override
	public void itemStateChanged(ItemEvent evt)
	{
		String language = "EN";
		if( getAppliConf() != null )
			language = getAppliConf().getLanguage();

		if( evt.getItem() instanceof JRadioButtonMenuItem )
		{
			JRadioButtonMenuItem btn = (JRadioButtonMenuItem) evt.getItem();
			if( btn.isSelected() )
			{
				if( !language.equals( btn.getText() ) )
				{
					try
					{
						changeLanguageInForms( btn.getText() );
						language = btn.getText();
					}
					catch( Throwable ex )
					{
						ex.printStackTrace();
						String languageTmp = language;
						language = btn.getText();
						setLanguageInMenu( languageTmp );
					}
				}
			}
		}
	}

	protected void changeLanguageInFormsIntern( String language, boolean withThrow ) throws Exception
	{
		getAppliConf().changeLanguage( language );
	}

	protected void changeLanguageInForms( String language ) throws Exception
	{
		String languageOld = getAppliConf().getLanguage();

		try
		{
			changeLanguageInFormsIntern( language, true );
		}
		catch( Throwable th )
		{
			th.printStackTrace();
			changeLanguageInFormsIntern( languageOld, false );
			throw( th );
		}
//		_language = language;
//		jRadioButtonMenuItemIdiomaOtro.setText(_additionalLanguage );
//		_language = language;
//		jRadioButtonMenuItemIdiomaOtro.setText(_additionalLanguage );
	}

	protected void setLanguageInMenu( String language )
	{
		try
		{
			if( ( language != null ) && ( _languageSubmenu != null ) )
			{
				MenuElement[] elems = _languageSubmenu.getSubElements();

				for( int ii=0; ii<elems.length; ii++ )
				{
					if( elems[ii] instanceof JRadioButtonMenuItem )
					{
						JRadioButtonMenuItem radioButton = (JRadioButtonMenuItem) elems[ii];
						if( language.equals( radioButton.getText() ) )
						{
							if( ! radioButton.isSelected() )
							{
								radioButton.setSelected(true);
							}
							break;
						}
					}
				}
			}
		}
		catch( Exception ex )
		{
			ex.printStackTrace();
		}
	}

	protected void setLanguageSubmenu( JPopupMenu languageSubmenu )
	{
		_languageSubmenu = languageSubmenu;

		updateLanguageSubmenu();
		updateLanguageInMenu();
	}

	public void addLanguageToMenu( String language )
	{
		GenericFunctions.instance().getObtainAvailableLanguages().newLanguageSetToConfiguration(language);
		updateLanguageSubmenu( );
		updateLanguageInMenu();
	}

	protected void updateLanguageInMenu()
	{
		String language = "EN";
		if( getAppliConf() != null )
			language = getAppliConf().getLanguage();
		setLanguageInMenu( language );
	}

	protected void updateLanguageSubmenu( )
	{
		updateLanguageSubmenu( _languageSubmenu );
		_initializing = false;
	}

	protected void updateLanguageSubmenu( JPopupMenu languageSubmenu )
	{
		ListOfLanguagesResult_int listsOfLanguages = GenericFunctions.instance().getObtainAvailableLanguages().getTotalListOfAvailableLanguages();
		updateLanguageSubmenu( languageSubmenu, listsOfLanguages );
	}

	protected void updateLanguageSubmenu( JPopupMenu languageSubmenu, ListOfLanguagesResult_int listsOfLanguages )
	{
		if( ! areListsOfAvailableLanguagesTheSameAsInTheMenu( languageSubmenu, listsOfLanguages) )
		{
			createNewLanguageSubmenu( languageSubmenu, listsOfLanguages );
		}
	}

	protected boolean areListsOfAvailableLanguagesTheSameAsInTheMenu( JPopupMenu languageSubmenu, ListOfLanguagesResult_int listsOfLanguages )
	{
		boolean result = false;

		if(  _initializing )
		{
			result = false;
		}
		else if( languageSubmenu != null )
		{
			String[] languages = null;
			if( listsOfLanguages != null )
			{
				languages = ArrayFunctions.getArrayJoiningLists( listsOfLanguages.getListOfFixedLanguages(), listsOfLanguages.getListOfOtherLanguages() );
			}

			MenuElement[] elems = languageSubmenu.getSubElements();

			if( ( languages == null ) && (elems.length==0) )
			{
				result = true;
			}
			else if( languages == null )
			{
				result = false;
			}
			else
			{
				result = true;
				int languageCount = 0;
				for( int ii=0; result && (ii<elems.length); ii++ )
				{
					if( elems[ii] instanceof JRadioButtonMenuItem )
					{
						JRadioButtonMenuItem radioButton = (JRadioButtonMenuItem) elems[ii];
						result = ( languages[languageCount].equals( radioButton.getText() ) );
						languageCount++;
					}
				}
				result = result && ( languageCount == languages.length );
			}
		}
		return( result );
	}

	protected void resetLanguageSubmenu( JPopupMenu languageSubmenu )
	{
		if( languageSubmenu != null )
		{
			if( _languageRadiobuttonGroup != null )
			{
				Enumeration<AbstractButton> enumeration = _languageRadiobuttonGroup.getElements();
				while( enumeration.hasMoreElements() )
				{
					_languageRadiobuttonGroup.remove( enumeration.nextElement() );
				}
			}
			else
			{
				_languageRadiobuttonGroup = new ButtonGroup();
			}

			languageSubmenu.removeAll();
		}
	}

	protected void createNewLanguageSubmenu( JPopupMenu languageSubmenu, ListOfLanguagesResult_int listsOfLanguages )
	{
		if( languageSubmenu != null )
		{
			resetLanguageSubmenu( languageSubmenu );

			List<String> fixedList = listsOfLanguages.getListOfFixedLanguages();
			List<String> otherList = listsOfLanguages.getListOfOtherLanguages();

			addListOfLanguages( languageSubmenu, _languageRadiobuttonGroup, fixedList );

			if( ( fixedList != null ) && ( fixedList.size() > 0 ) &&
				( otherList != null ) && ( otherList.size() > 0 ) )
			{
				languageSubmenu.addSeparator();
			}

			addListOfLanguages( languageSubmenu, _languageRadiobuttonGroup, otherList );
		}
	}

	protected void addListOfLanguages( JPopupMenu languageSubmenu, ButtonGroup bg, List<String> listOfLanguages )
	{
		if( ( languageSubmenu != null ) && ( listOfLanguages != null ) )
		{
			Iterator<String> it = listOfLanguages.iterator();
			while( it.hasNext() )
			{
				JRadioButtonMenuItem rbmi = new JRadioButtonMenuItem();
				rbmi.setText( it.next() );
				bg.add(rbmi);
				languageSubmenu.add( rbmi );
				rbmi.addItemListener(this);
			}
		}
	}
}
