/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.desktop.view.about;

import com.frojasg1.general.StringFunctions;
import com.frojasg1.general.desktop.mouse.AvailableCursors;
import com.frojasg1.general.desktop.mouse.MouseFunctions;
import com.frojasg1.general.screen.ScreenFunctions;
import com.frojasg1.general.desktop.view.editorkits.WrapEditorKit;
import com.frojasg1.general.desktop.view.pdf.ImageJPanel;
import com.frojasg1.desktop.liblens.graphics.lens.LensJPanel;
import java.awt.Color;
import java.awt.Desktop;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.image.BufferedImage;
import java.net.URI;
import javax.swing.JEditorPane;
import javax.swing.JScrollPane;
import javax.swing.JViewport;
import javax.swing.text.Document;
import javax.swing.text.JTextComponent;

/**
 *
 * @author Usuario
 */
public class GenericAboutJPanel extends javax.swing.JPanel implements MouseListener, MouseMotionListener
{
	protected static final String SPACES_LINE = "                                                                      " + StringFunctions.RETURN;

	protected AboutJPanelControllerInterface _controller = null;

	protected ImageJPanel _imageJPanel = null;

	protected BufferedImage _image = null;

	protected JEditorPane _rtfEditorPane = null;

	protected LensJPanel _lensJPanel = null;

	protected String _lastURL = null;

	protected GenericAboutJPanel _this = null;

//	protected int _oldScrollYYValue = -1;

	/**
	 * Creates new form GenericAboutJPanel
	 */
	public GenericAboutJPanel( AboutJPanelControllerInterface controller, BufferedImage image )
	{
		_this = this;
		_controller = controller;
		setImage( image );

		initComponents();

		initOwnComponents();
	}

	public void setSendEmailButtonText( String text )
	{
		jBtn_sendEmail.setText( text );
	}

	public void setLensJPanel( LensJPanel ljp )
	{
		_lensJPanel = ljp;

		if( _lensJPanel != null )
		{
			_lensJPanel.M_addComponentNotToPaint( jTa_text );
//			_lensJPanel.M_addComponentNotToPaint( jBtn_exit );
//			_lensJPanel.M_addComponentNotToPaint( jBtn_sendEmail );
			_lensJPanel.M_addComponentNotToPaint( getJTextComponent() );
		}
	}

	public void setImage( BufferedImage image )
	{
		if( ( image == null ) || ( image.getHeight() == 0 ) )
		{
			_image = new BufferedImage( 245, 120, BufferedImage.TYPE_INT_RGB );
			Graphics gc = _image.getGraphics();
			gc.setColor( Color.WHITE );
			gc.fillRect( 0, 0, 245, 120 );
		}
		else
			_image = image;
	}

	public void setText( String text )
	{
		jTa_text.setText( text );
		jTa_text.setSelectionStart(0);
		jTa_text.setSelectionEnd(0);
	}

	public JTextComponent getJTextComponent()
	{
		JTextComponent result = _rtfEditorPane;
		if( result == null )
			result = jTa_text;

		return( result );
	}

	public void setRtfDocument( Document doc )
	{
		if( doc != null )
		{
			jTa_text.setFocusable( false );
			if( _rtfEditorPane != null )
				_lensJPanel.removeComponentNotToPaint( _rtfEditorPane );

			_rtfEditorPane = new JEditorPane( "text/rtf", "" );
			_rtfEditorPane.setEditorKit( new WrapEditorKit() );
			_rtfEditorPane.setEditable( false );
			_rtfEditorPane.setDocument( doc );
			jScrollPane1.setViewportView( _rtfEditorPane );
			_rtfEditorPane.setSelectionStart(0);
			_rtfEditorPane.setSelectionEnd(0);

			try
			{
				doc.insertString( doc.getLength(), StringFunctions.RETURN, null);
				doc.insertString( doc.getLength(), SPACES_LINE, null);
				doc.insertString( doc.getLength(), SPACES_LINE, null);
				doc.insertString( doc.getLength(), SPACES_LINE, null);
			}
			catch( Throwable th )
			{
				th.printStackTrace();
			}

			if( _lensJPanel != null )
			{
				_lensJPanel.M_addComponentNotToPaint( _rtfEditorPane );
				_rtfEditorPane.requestFocusInWindow();
			}
		}
		else
		{
			_rtfEditorPane = null;
			jScrollPane1.setViewportView( jTa_text );
			jTa_text.setFocusable( true );
			jTa_text.requestFocusInWindow();
			jTa_text.setSelectionStart(0);
			jTa_text.setSelectionEnd(0);
		}
		_lensJPanel.repaint();
	}

	public void initOwnComponents()
	{
		jTa_text.setWrapStyleWord(true);
		jTa_text.setLineWrap( true );

		_imageJPanel = new ImageJPanel( _image );
		_imageJPanel.setFrameThick( 3 );

		jPanelImage.add( _imageJPanel );

		int width = (int) ( jPanelImage.getHeight() * _image.getWidth() / _image.getHeight() );
		int height = (int) jPanelImage.getHeight();

		Rectangle jpiBounds = jPanelImage.getBounds();
		jPanelImage.setBounds( (int) ( jpiBounds.getX() + ( jpiBounds.getWidth() - width ) / 2 ),
								(int) ( jpiBounds.getY() + ( jpiBounds.getHeight() - height ) / 2 ),
								width, height );
		_imageJPanel.setBounds( 0, 0, width, height );
		_imageJPanel.setPreferredSize( new Dimension( width, height ) );

		jTa_text.addMouseListener(this);
		jTa_text.addMouseMotionListener(this);

		jScrollPane1.getViewport().addMouseListener( this );
		jScrollPane1.getViewport().addMouseMotionListener( this );
	}

	public void setFrameThick( int value )
	{
		_imageJPanel.setFrameThick(value);
	}
	
	public void setFontForTextLines( Font font )
	{
		_imageJPanel.setFontForTextLines(font);
	}

	public void setForegroundColorForTextLines( Color color )
	{
		_imageJPanel.setForegroundColorForTextLines(color);
	}
	
	public void addTextLine( ImageJPanel.TextLine textLine )
	{
		_imageJPanel.addTextLine(textLine);
	}

	public void clearTextLines()
	{
		_imageJPanel.clearTextLines();
	}

	public Dimension getSizeOfImageJPanel()
	{
		return( _imageJPanel.getSize() );
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	@SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jBtn_exit = new javax.swing.JButton();
        jScrollPane1 = new javax.swing.JScrollPane();
        jTa_text = new javax.swing.JTextArea();
        jBtn_sendEmail = new javax.swing.JButton();
        jPanelImage = new javax.swing.JPanel();

        setPreferredSize(new java.awt.Dimension(650, 405));
        setLayout(null);

        jBtn_exit.setText("Exit");
        jBtn_exit.setName("jBtn_exit"); // NOI18N
        jBtn_exit.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jBtn_exitActionPerformed(evt);
            }
        });
        add(jBtn_exit);
        jBtn_exit.setBounds(540, 100, 100, 20);

        jTa_text.setEditable(false);
        jTa_text.setColumns(20);
        jTa_text.setRows(5);
        jScrollPane1.setViewportView(jTa_text);

        add(jScrollPane1);
        jScrollPane1.setBounds(10, 130, 630, 230);

        jBtn_sendEmail.setText("Send e-mail to:  frojasg1@hotmail.com");
        jBtn_sendEmail.setFocusable(false);
        jBtn_sendEmail.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jBtn_sendEmailActionPerformed(evt);
            }
        });
        add(jBtn_sendEmail);
        jBtn_sendEmail.setBounds(10, 370, 630, 25);

        jPanelImage.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));

        javax.swing.GroupLayout jPanelImageLayout = new javax.swing.GroupLayout(jPanelImage);
        jPanelImage.setLayout(jPanelImageLayout);
        jPanelImageLayout.setHorizontalGroup(
            jPanelImageLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 108, Short.MAX_VALUE)
        );
        jPanelImageLayout.setVerticalGroup(
            jPanelImageLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 108, Short.MAX_VALUE)
        );

        add(jPanelImage);
        jPanelImage.setBounds(260, 10, 110, 110);
    }// </editor-fold>//GEN-END:initComponents

    private void jBtn_exitActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jBtn_exitActionPerformed
        // TODO add your handling code here:

        _controller.exit(this);

    }//GEN-LAST:event_jBtn_exitActionPerformed

    private void jBtn_sendEmailActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jBtn_sendEmailActionPerformed
        // TODO add your handling code here:

		_controller.sendEmailAddress();
/*
        try
        {
            Desktop.getDesktop().mail( new URI( "mailto:" + _controller.getEmailAddress() ) );
        }
        catch( Throwable th )
        {
            th.printStackTrace();
        }
*/

    }//GEN-LAST:event_jBtn_sendEmailActionPerformed


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jBtn_exit;
    private javax.swing.JButton jBtn_sendEmail;
    private javax.swing.JPanel jPanelImage;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JTextArea jTa_text;
    // End of variables declaration//GEN-END:variables

	@Override
	public void mouseClicked(MouseEvent e)
	{
		if( (_lastURL != null) && Desktop.isDesktopSupported() )
		{
			try
			{
				Desktop.getDesktop().browse( new URI( _lastURL ) );
			}
			catch( Throwable th )
			{
				th.printStackTrace();
			}
		}
	}

	@Override
	public void mousePressed(MouseEvent e)
	{
	}

	@Override
	public void mouseReleased(MouseEvent e)
	{
	}

	@Override
	public void mouseEntered(MouseEvent e)
	{
	}

	@Override
	public void mouseExited(MouseEvent e)
	{
	}

	@Override
	public void mouseDragged(MouseEvent e)
	{
		moveMouse(e);
	}

	@Override
	public void mouseMoved(MouseEvent e)
	{
		moveMouse(e);
	}

	public void moveMouse( MouseEvent e )
	{
		if( e.getSource() instanceof JViewport )
		{
			JViewport jvp = (JViewport) e.getSource();
			changeUrlCursor( jvp, -1 );
		}
	}

	protected void changeUrlCursor( JViewport textComponentParent, int oldScrollYYValue )
	{
		if( textComponentParent.getComponent(0) instanceof JTextComponent )
		{
			JTextComponent jtc = (JTextComponent) textComponentParent.getComponent(0);

			Point pt = MouseFunctions.getMouseLocation();
			if( ScreenFunctions.isInsideComponent( textComponentParent, pt ) )
			{
				Point parentPosition = textComponentParent.getLocationOnScreen();

				if( textComponentParent.getParent() instanceof JScrollPane )
				{
					JScrollPane parent = (JScrollPane) textComponentParent.getParent();

					int scrollYYValue = parent.getVerticalScrollBar().getValue();
					
					pt = new Point( (int) Math.min( jtc.getWidth()-1,
													(pt.getX() - parentPosition.getX()) ),
									(int) Math.min( jtc.getHeight()-1,
													(double) (  ( pt.getY() - parentPosition.getY() ) *
															 parent.getVerticalScrollBar().getVisibleAmount()) /
														textComponentParent.getHeight() ) +
														scrollYYValue );
					int pos = jtc.viewToModel(pt);

					_lastURL = getCurrentUrl( jtc, pos );

//					System.out.println( "pt:" + pt + " position: " + pos + "  url:" + _lastURL );

					boolean cursorChanged = false;
					if( ( _lastURL != null ) && !jtc.getCursor().getName().equals( AvailableCursors._handCursor.getName() ) )
					{
						jScrollPane1.getViewport().getComponent(0).setCursor( AvailableCursors._handCursor );
						cursorChanged = true;
					}
					else if( (_lastURL == null ) && !jtc.getCursor().getName().equals( AvailableCursors._defaultCursor.getName() ) )
					{
						jScrollPane1.getViewport().getComponent(0).setCursor( AvailableCursors._defaultCursor );
						cursorChanged = true;
					}
					
					if( cursorChanged && ( oldScrollYYValue != -1 ) &&
						( oldScrollYYValue != scrollYYValue ) )
					{
						jScrollPane1.getVerticalScrollBar().setValue( oldScrollYYValue );
						jScrollPane1.getVerticalScrollBar().setValue( scrollYYValue );
						java.awt.EventQueue.invokeLater(new Runnable() {
							public void run() {
								_lensJPanel.repaint();
							}
						});
					}
				}
			}
		}
	}

	public void changeUrlCursor( int oldScrollYYValue )
	{
		changeUrlCursor(jScrollPane1.getViewport(), oldScrollYYValue );
	}

	protected String getCurrentUrl( JTextComponent tc, int pos )
	{
		String result = null;

		String text = null;

		try
		{
			text = tc.getDocument().getText( 0, tc.getDocument().getLength() );
		}
		catch( Throwable th )
		{
			th.printStackTrace();
			text = "";
		}

		int initialPosition = StringFunctions.lastIndexOfAnyChar( text, " \t\r\n", pos );
		initialPosition++;

		int finalPosition = StringFunctions.indexOfAnyChar( text, " \t\r\n", pos );
		if( finalPosition == -1 )
			finalPosition = text.length();

		if( (finalPosition - initialPosition) > 4 )
		{
			String word = text.substring( initialPosition, finalPosition );

			if( ( word.length() > 4 ) && word.substring( 0, 4 ).equals( "http" ) )
				result = word;
		}

		return( result );
	}
}
