/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.desktop.view.panels;

import com.frojasg1.general.desktop.view.ViewFunctions;
import com.hotmail.frojasg1.applications.common.components.resizecomp.MapResizeRelocateComponentItem;
import com.hotmail.frojasg1.applications.common.components.resizecomp.ResizableComponentInterface;
import com.hotmail.frojasg1.applications.common.components.resizecomp.ResizeRelocateItem;
import java.awt.Insets;
import javax.swing.JButton;
import javax.swing.SwingUtilities;

/**
 *
 * @author Usuario
 */
public class AcceptCancelRevertPanel extends javax.swing.JPanel
									implements ResizableComponentInterface, InformerInterface
{
	protected static final String _resourcePath = "com/frojasg1/generic/resources/acceptcancel";

	protected AcceptCancelRevertControllerInterface _controller = null;

	protected MapResizeRelocateComponentItem _mapRRCI = null;


	/**
	 * Creates new form AcceptCancelRevertPanel
	 */
	public AcceptCancelRevertPanel( AcceptCancelRevertControllerInterface controller )
	{
		_controller = controller;

		initComponents();
		
		setBounds( 0, 0, jPanel1.getWidth(), jPanel1.getHeight() );
		
		initNavigatorButtons();

		_mapRRCI = createResizeRelocateInfo();
	}

	protected String getResourceName( String shortResourceName )
	{
		return( _resourcePath + "/" + shortResourceName );
	}

	public JButton getCancelButton()
	{
		return( jB_cancel );
	}

	protected void initNavigatorButtons()
	{
		Insets insets = new Insets( 1, 1, 1, 1 );

		ViewFunctions.addImageToButton( jB_cancel, getResourceName( "erase.png" ), insets );
		ViewFunctions.addImageToButton( jB_acceptChanges, getResourceName( "accept.png" ), insets );
		ViewFunctions.addImageToButton( jB_revertChanges, getResourceName( "revert.png" ), insets );
	}

	public MapResizeRelocateComponentItem getResizeRelocateInfo()
	{
		if( _mapRRCI == null )
		{
			_mapRRCI = createResizeRelocateInfo();
		}

		return( _mapRRCI );
	}

	protected MapResizeRelocateComponentItem createResizeRelocateInfo()
	{
		MapResizeRelocateComponentItem mapRRCI = new MapResizeRelocateComponentItem();

		try
		{
			mapRRCI.putResizeRelocateComponentItem( jPanel1, ResizeRelocateItem.RESIZE_TO_RIGHT +
															ResizeRelocateItem.RESIZE_TO_BOTTOM );
			mapRRCI.putResizeRelocateComponentItem( jB_acceptChanges, ResizeRelocateItem.MOVE_LEFT_SIDE_PROPORTIONAL +
																		ResizeRelocateItem.MOVE_RIGHT_SIDE_PROPORTIONAL +
																		ResizeRelocateItem.MOVE_TOP_SIDE_PROPORTIONAL +
																		ResizeRelocateItem.MOVE_BOTTOM_SIDE_PROPORTIONAL );
//			mapRRCI.putResizeRelocateComponentItem(jB_cancel, ResizeRelocateItem.MOVE_LEFT_SIDE_PROPORTIONAL +
//																		ResizeRelocateItem.MOVE_RIGHT_SIDE_PROPORTIONAL +
//																		ResizeRelocateItem.MOVE_TOP_SIDE_PROPORTIONAL +
//																		ResizeRelocateItem.MOVE_BOTTOM_SIDE_PROPORTIONAL );
			mapRRCI.putResizeRelocateComponentItem( jB_revertChanges, ResizeRelocateItem.MOVE_LEFT_SIDE_PROPORTIONAL +
																		ResizeRelocateItem.MOVE_RIGHT_SIDE_PROPORTIONAL +
																		ResizeRelocateItem.MOVE_TOP_SIDE_PROPORTIONAL +
																		ResizeRelocateItem.MOVE_BOTTOM_SIDE_PROPORTIONAL );

//			HintForComponent hfc = new HintForComponent( jB_acceptChanges, "Accept changes" );
//			HintForComponent hfc2 = new HintForComponent( jB_eraseTag, "Erase record" );
//			HintForComponent hfc3 = new HintForComponent( jB_revertChanges, "Revert changes" );
		}
		catch( Throwable th )
		{
			th.printStackTrace();
		}

		return( mapRRCI );
	}

	public void setAcceptButtonEnabled( boolean value )
	{
		jB_acceptChanges.setEnabled( value );
	}

	public void setRevertButtonEnabled( boolean value )
	{
		jB_revertChanges.setEnabled( value );
	}

	public void setEraseButtonEnabled( boolean value )
	{
		jB_cancel.setEnabled( value );
	}

	public void setNameForAcceptButton( String name )
	{
		this.jB_acceptChanges.setName( name );
	}
	
	public void setNameForRevertButton( String name )
	{
		this.jB_revertChanges.setName( name );
	}
	
	public void setNameForEraseButton( String name )
	{
		this.jB_cancel.setName( name );
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	@SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel1 = new javax.swing.JPanel();
        jB_cancel = new javax.swing.JButton();
        jB_revertChanges = new javax.swing.JButton();
        jB_acceptChanges = new javax.swing.JButton();

        setMinimumSize(new java.awt.Dimension(130, 50));
        setPreferredSize(new java.awt.Dimension(130, 50));
        setLayout(null);

        jPanel1.setLayout(null);

        jB_cancel.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jB_cancel.setMaximumSize(new java.awt.Dimension(30, 30));
        jB_cancel.setMinimumSize(new java.awt.Dimension(30, 30));
        jB_cancel.setName("jB_cancel"); // NOI18N
        jB_cancel.setPreferredSize(new java.awt.Dimension(30, 30));
        jB_cancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jB_cancelActionPerformed(evt);
            }
        });
        jPanel1.add(jB_cancel);
        jB_cancel.setBounds(10, 10, 30, 30);

        jB_revertChanges.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jB_revertChanges.setMaximumSize(new java.awt.Dimension(30, 30));
        jB_revertChanges.setMinimumSize(new java.awt.Dimension(30, 30));
        jB_revertChanges.setName("jB_revertChanges"); // NOI18N
        jB_revertChanges.setPreferredSize(new java.awt.Dimension(30, 30));
        jB_revertChanges.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jB_revertChangesActionPerformed(evt);
            }
        });
        jPanel1.add(jB_revertChanges);
        jB_revertChanges.setBounds(90, 10, 30, 30);

        jB_acceptChanges.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jB_acceptChanges.setMaximumSize(new java.awt.Dimension(30, 30));
        jB_acceptChanges.setMinimumSize(new java.awt.Dimension(30, 30));
        jB_acceptChanges.setName("jB_acceptChanges"); // NOI18N
        jB_acceptChanges.setPreferredSize(new java.awt.Dimension(30, 30));
        jB_acceptChanges.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jB_acceptChangesActionPerformed(evt);
            }
        });
        jPanel1.add(jB_acceptChanges);
        jB_acceptChanges.setBounds(50, 10, 30, 30);

        add(jPanel1);
        jPanel1.setBounds(0, 0, 130, 50);
    }// </editor-fold>//GEN-END:initComponents

    private void jB_cancelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jB_cancelActionPerformed
        // TODO add your handling code here:

//		System.out.println( "jB_cancel.getBounds()" + jB_cancel.getBounds() );
//		System.out.println( "jB_acceptChanges.getBounds()" + jB_acceptChanges.getBounds() );
//		System.out.println( "jB_revertChanges.getBounds()" + jB_revertChanges.getBounds() );

		if( _controller != null )
			_controller.cancel( this );

    }//GEN-LAST:event_jB_cancelActionPerformed

    private void jB_acceptChangesActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jB_acceptChangesActionPerformed
        // TODO add your handling code here:

		if( _controller != null )
			_controller.accept( this );

    }//GEN-LAST:event_jB_acceptChangesActionPerformed

    private void jB_revertChangesActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jB_revertChangesActionPerformed
        // TODO add your handling code here:

		if( _controller != null )
			_controller.revert( this );

    }//GEN-LAST:event_jB_revertChangesActionPerformed


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jB_acceptChanges;
    private javax.swing.JButton jB_cancel;
    private javax.swing.JButton jB_revertChanges;
    private javax.swing.JPanel jPanel1;
    // End of variables declaration//GEN-END:variables

	@Override
	public void doTasksAfterResizingComponent()
	{
		initNavigatorButtons();		// to resize the images.
	}
}
